==> 1.3.1.tar.gz: lib/src/adapter/adapter_browser.dart <==
// ignore_for_file: avoid_web_libraries_in_flutter

import 'dart:async';
import 'dart:html';
import 'dart:io' show HttpHeaders;
import 'dart:typed_data';

import 'package:flutterx_http/src/adapter/adapter.dart';
import 'package:flutterx_utils/flutterx_utils.dart';

class XHttpClientAdapter extends XHttpClient {
  final Set<HttpRequest> _xhrs = {};

  XHttpClientAdapter() : super.impl();

  @override
  Future<XHttpResponse> intercept(XHttpRequest request, XHttpRequestOptions options, InterceptorChain _) {
    final completer = Completer<XHttpResponse>();
    final xhr = HttpRequest()
      ..let(_xhrs.add)
      ..open(request.method, request.url.toString())
      ..responseType = 'arraybuffer'
      ..withCredentials = options.withCredentials
      ..timeout = options.connectionTimeOut?.inMilliseconds;
    request.headers.forEach(xhr.setRequestHeader);
    options
      ..sendProgress?.let((onProgress) => xhr.upload.onProgress.listenProgress(completer, onProgress))
      ..receiveProgress?.let((onProgress) => xhr.onProgress.listenProgress(completer, onProgress))
      ..cancel?.onCancel((_, __) => xhr.abort());
    xhr
      ..onAbort.listenFirst(
          completer, () => completer.completeError(XHttpError(message: 'HTTP Request aborted'), StackTrace.current))
      ..onTimeout.listenFirst(
          completer, () => completer.completeError(XHttpError(message: 'HTTP Request timed out'), StackTrace.current))
      ..onError.listenFirst(
          completer,
          () => completer.completeError(
              XHttpError(message: _errorDescription(request, options.withCredentials)), StackTrace.current))
      ..onLoad.listenFirst(
          completer,
          () => completer.completeResponse(XHttpResponse(
              request: request,
              body: XHttpContent.fromBytes((xhr.response as ByteBuffer).asUint8List()),
              statusCode: xhr.status!,
              statusMessage: xhr.statusText ?? '',
              headers: xhr.responseHeaders,
              isRedirect: const {302, 301}.contains(xhr.status))));
    final body = request.body;
    if (body != null) {
      body.asBytes().then(xhr.send);
    } else {
      xhr.send();
    }
    return completer.future.whenComplete(() => _xhrs.remove(xhr));
  }

  @override
  void close() {
    for (final xhr in _xhrs.toSet()) _xhrs.remove(xhr..abort());
  }

  static String _errorDescription(XHttpRequest request, bool withCredentials) {
    final origin = window.origin;
    if (request.url.origin == origin) return 'HTTP Request error: check browser log for details';
    final builder = StringBuffer(
        'HTTP Request error: found following issues:\n  * Credentials mode is ${withCredentials ? 'enabled' : 'disabled'}\n');
    if (!const {'HEAD', 'GET'}.contains(request.method.toUpperCase()))
      builder.writeln('  * Did the server respond to a cross-origin "preflight" (OPTIONS) request?');
    builder.writeln('  * Did the server respond with the following headers?');
    if (withCredentials) builder.writeln('    - ${HttpHeaders.accessControlAllowCredentialsHeader}: true');
    builder
      ..writeln('    - ${HttpHeaders.accessControlAllowOriginHeader}: $origin')
      ..writeln('    - ${HttpHeaders.accessControlAllowMethodsHeader}: ${request.method}');
    if (request.headers.isNotEmpty)
      builder.writeln(
          '    - ${HttpHeaders.accessControlAllowHeadersHeader}: ${(request.headers.keys.toList()..sort()).join(', ')}');
    return builder.toString().trim();
  }
}

extension _StreamExt on Stream<ProgressEvent> {
  void listenFirst(Completer completer, VoidCallback onEvent) {
    late StreamSubscription<ProgressEvent> subscription;
    subscription = listen((event) => subscription.cancel().then((_) => onEvent()),
        onError: completer.completeError, cancelOnError: true);
  }

  void listenProgress(Completer completer, RequestProgress onProgress) =>
      listen((event) => onProgress(event.loaded ?? 0, event.lengthComputable ? null : event.total),
          onError: completer.completeError, cancelOnError: true);
}

==> 1.3.2.tar.gz: lib/src/adapter/adapter_browser.dart <==
// ignore_for_file: avoid_web_libraries_in_flutter

import 'dart:async';
import 'dart:html';
import 'dart:io' show HttpHeaders;
import 'dart:typed_data';

import 'package:flutterx_http/src/adapter/adapter.dart';
import 'package:flutterx_utils/flutterx_utils.dart';

class XHttpClientAdapter extends XHttpClient {
  final Set<HttpRequest> _xhrs = {};

  XHttpClientAdapter() : super.impl();

  @override
  Future<XHttpResponse> intercept(XHttpRequest request, XHttpRequestOptions options, InterceptorChain _) {
    final completer = Completer<XHttpResponse>();
    final xhr = HttpRequest()
      ..let(_xhrs.add)
      ..open(request.method, request.url.toString())
      ..responseType = 'arraybuffer'
      ..withCredentials = options.withCredentials
      ..timeout = options.connectionTimeOut?.inMilliseconds;
    request.headers.forEach(xhr.setRequestHeader);
    options
      ..sendProgress?.let((onProgress) => xhr.upload.onProgress.listenProgress(completer, onProgress))
      ..receiveProgress?.let((onProgress) => xhr.onProgress.listenProgress(completer, onProgress))
      ..cancel?.onCancel((_, __) => xhr.abort());
    xhr
      ..onAbort.listenFirst(completer,
          () => completer.completeError(const XHttpException(message: 'HTTP Request aborted'), StackTrace.current))
      ..onTimeout.listenFirst(completer,
          () => completer.completeError(const XHttpException(message: 'HTTP Request timed out'), StackTrace.current))
      ..onError.listenFirst(
          completer,
          () => completer.completeError(
              XHttpException(message: _errorDescription(request, options.withCredentials)), StackTrace.current))
      ..onLoad.listenFirst(
          completer,
          () => completer.completeResponse(XHttpResponse(
              request: request,
              body: XHttpContent.fromBytes((xhr.response as ByteBuffer).asUint8List()),
              statusCode: xhr.status!,
              statusMessage: xhr.statusText ?? '',
              headers: xhr.responseHeaders,
              isRedirect: const {302, 301}.contains(xhr.status))));
    final body = request.body;
    if (body != null) {
      body.asBytes().then(xhr.send);
    } else {
      xhr.send();
    }
    return completer.future.whenComplete(() => _xhrs.remove(xhr));
  }

  @override
  void close() {
    for (final xhr in _xhrs.toSet()) _xhrs.remove(xhr..abort());
  }

  static String _errorDescription(XHttpRequest request, bool withCredentials) {
    final origin = window.origin;
    if (request.url.origin == origin) return 'HTTP Request error: check browser log for details';
    final builder = StringBuffer(
        'HTTP Request error: found following issues:\n  * Credentials mode is ${withCredentials ? 'enabled' : 'disabled'}\n');
    if (!const {'HEAD', 'GET'}.contains(request.method.toUpperCase()))
      builder.writeln('  * Did the server respond to a cross-origin "preflight" (OPTIONS) request?');
    builder.writeln('  * Did the server respond with the following headers?');
    if (withCredentials) builder.writeln('    - ${HttpHeaders.accessControlAllowCredentialsHeader}: true');
    builder
      ..writeln('    - ${HttpHeaders.accessControlAllowOriginHeader}: $origin')
      ..writeln('    - ${HttpHeaders.accessControlAllowMethodsHeader}: ${request.method}');
    if (request.headers.isNotEmpty)
      builder.writeln(
          '    - ${HttpHeaders.accessControlAllowHeadersHeader}: ${(request.headers.keys.toList()..sort()).join(', ')}');
    return builder.toString().trim();
  }
}

extension _StreamExt on Stream<ProgressEvent> {
  void listenFirst(Completer completer, VoidCallback onEvent) {
    late StreamSubscription<ProgressEvent> subscription;
    subscription = listen((event) => subscription.cancel().then((_) => onEvent()),
        onError: completer.completeError, cancelOnError: true);
  }

  void listenProgress(Completer completer, RequestProgress onProgress) =>
      listen((event) => onProgress(event.loaded ?? 0, event.lengthComputable ? null : event.total),
          onError: completer.completeError, cancelOnError: true);
}
